//------------------------------------------------------------------------------
// Copyright (c) 2008 www.dnaide.com
// Licensed under the MIT (MIT-LICENSE.txt)
//------------------------------------------------------------------------------
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Configuration;
using DNAide.Web; 

namespace DNAide.Configuration
{
	public class HttpCompressionModuleMimeTypeConfig : ConfigurationElement
	{

		[ConfigurationProperty("enabled", DefaultValue = true, IsRequired = false)]
		public Boolean Enabled
		{
			get
			{
				return (bool)this["enabled"];
			}
			set
			{
				this["enabled"] = value;
			}
		}

		[ConfigurationProperty("mimeType", IsRequired = true)]
		public String MimeType
		{
			get
			{
				return (string)this["mimeType"];
			}
			set
			{
				this["mimeType"] = value;
			}
		}


		/// <summary>
		/// Collection of rules
		/// </summary>
		[ConfigurationProperty("rules", IsRequired = false, IsDefaultCollection = true)]
		public HttpCompressionModuleRuleConfigCollection Rules
		{
			get
			{
				return (HttpCompressionModuleRuleConfigCollection)this["rules"];
			}
		}
	}


	/// <summary>HttpCompressionModuleMimeTypeConfigCollection</summary>
	public class HttpCompressionModuleMimeTypeConfigCollection : ConfigurationElementCollection, ICollection<HttpCompressionModuleMimeTypeConfig>
	{
		/// <summary>IsReadOnly</summary>
		public new bool IsReadOnly
		{
			get
			{
				return base.IsReadOnly();
			}
		}

		/// <summary>HttpCompressionModuleMimeTypeConfig</summary>
		public HttpCompressionModuleMimeTypeConfig this[int index]
		{
			get
			{
				return (HttpCompressionModuleMimeTypeConfig)BaseGet(index);
			}
			set
			{
				if (BaseGet(index) != null)
				{
					BaseRemoveAt(index);
				}
				BaseAdd(index, value);
			}
		}

		/// <summary>HttpCompressionModuleMimeTypeConfig</summary>
		new public HttpCompressionModuleMimeTypeConfig this[string mimeType]
		{
			get
			{
				return (HttpCompressionModuleMimeTypeConfig)BaseGet(mimeType);
			}
		}

		/// <summary>HttpCompressionModuleMimeTypeConfig</summary>
		public HttpCompressionModuleMimeTypeConfigCollection()
		{
		}

		/// <summary>Add</summary>
		public void Add(HttpCompressionModuleMimeTypeConfig item)
		{
			if (item == null)
				return;

			BaseAdd(item);
		}

		/// <summary>Clear</summary>
		public void Clear()
		{
			BaseClear();
		}

		/// <summary>Contains</summary>
		public bool Contains(HttpCompressionModuleMimeTypeConfig item)
		{
			if (BaseIndexOf(item) >= 0)
				return true;
			else
				return false;
		}

		/// <summary>CopyTo</summary>
		public void CopyTo(HttpCompressionModuleMimeTypeConfig[] array, int arrayIndex)
		{
			((ICollection)this).CopyTo(array, arrayIndex);
		}

		/// <summary>CreateNewElement</summary>
		protected override ConfigurationElement CreateNewElement()
		{
			return new HttpCompressionModuleMimeTypeConfig();
		}

		/// <summary>IndexOf</summary>
		public int IndexOf(HttpCompressionModuleMimeTypeConfig item)
		{
			if (item == null)
				return -1;

			return BaseIndexOf(item);
		}

		/// <summary>GetElementKey</summary>
		protected override object GetElementKey(ConfigurationElement element)
		{
			return ((HttpCompressionModuleMimeTypeConfig)element).MimeType;
		}

		/// <summary>GetEnumerator</summary>
		public new IEnumerator<HttpCompressionModuleMimeTypeConfig> GetEnumerator()
		{
			for (int i = 0; i < Count; i++)
			{
				yield return (HttpCompressionModuleMimeTypeConfig)this[i];
			}
		}

		/// <summary>Remove</summary>
		public bool Remove(HttpCompressionModuleMimeTypeConfig item)
		{
			if (item == null)
				return false;

			if (BaseIndexOf(item) >= 0)
				BaseRemove(item.MimeType);

			return true;
		}

		/// <summary>Remove</summary>
		public void Remove(string mimeType)
		{
			BaseRemove(mimeType);
		}

		/// <summary>RemoveAt</summary>
		public void RemoveAt(int index)
		{
			BaseRemoveAt(index);
		}

	}
}
