//------------------------------------------------------------------------------
// Copyright (c) 2008 www.dnaide.com
// Licensed under the MIT (MIT-LICENSE.txt)
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Text;
using DNAide.Web.UI.Scripts;

namespace DNAide.Configuration
{
	public class ScriptAideConfig : ConfigurationElement
	{

		[ConfigurationProperty("regexCompilationMode", DefaultValue = "Compiled", IsRequired = false)]
		public RegexCompilationType RegexCompilationMode
		{
			get
			{
				return (RegexCompilationType)this["regexCompilationMode"];
			}
			set
			{
				this["regexCompilationMode"] = value;
			}
		}

		[ConfigurationProperty("defaultObfuscateType", DefaultValue = ScriptObfuscateType.None, IsRequired = false)]
		public ScriptObfuscateType DefaultObfuscateType
		{
			get
			{
				return (ScriptObfuscateType)this["defaultObfuscateType"];
			}
			set
			{
				this["defaultObfuscateType"] = value;
			}
		}

		[ConfigurationProperty("defaultRegisterLocationType", DefaultValue = ScriptRegisterLocationType.Head, IsRequired = false)]
		public ScriptRegisterLocationType DefaultRegisterLocationType
		{
			get
			{
				return (ScriptRegisterLocationType)this["defaultRegisterLocationType"];
			}
			set
			{
				this["defaultRegisterLocationType"] = value;
			}
		}

		[ConfigurationProperty("defaultControl", DefaultValue = "", IsRequired = false)]
		public String DefaultControl
		{
			get
			{
				return (String)this["defaultControl"];
			}
			set
			{
				this["defaultControl"] = value;
			}
		}


		[ConfigurationProperty("mimeType", DefaultValue = "application/x-javascript", IsRequired = false)]
		public String MimeType
		{
			get
			{
				return (String)this["mimeType"];
			}
			set
			{
				this["mimeType"] = value;
			}
		}


		[ConfigurationProperty("batchIncludes", DefaultValue = false, IsRequired = false)]
		public Boolean BatchIncludes
		{
			get
			{
				return (bool)this["batchIncludes"];
			}
			set
			{
				this["batchIncludes"] = value;
			}
		}


		[ConfigurationProperty("useCompressionHandler", DefaultValue = false, IsRequired = false)]
		public Boolean UseCompressionHandler
		{
			get
			{
				return (bool)this["useCompressionHandler"];
			}
			set
			{
				this["useCompressionHandler"] = value;
			}
		}

		/// <summary>
		/// Collection of replacement rules
		/// </summary>
		[ConfigurationProperty("globalReplacementRules", IsRequired = false, IsDefaultCollection = true)]
		public ScriptAideReplacementRuleConfigCollection GlobalReplacementRules
		{
			get
			{
				return (ScriptAideReplacementRuleConfigCollection)this["globalReplacementRules"];
			}
		}

	}


	/// <summary>ScriptAideReplacementRuleConfigCollection</summary>
	public class ScriptAideReplacementRuleConfigCollection : ConfigurationElementCollection, ICollection<ScriptAideReplacementRuleConfig>
	{
		/// <summary>IsReadOnly</summary>
		public new bool IsReadOnly
		{
			get
			{
				return base.IsReadOnly();
			}
		}

		/// <summary>HttpCompressionModuleMimeTypeConfig</summary>
		public ScriptAideReplacementRuleConfig this[int index]
		{
			get
			{
				return (ScriptAideReplacementRuleConfig)BaseGet(index);
			}
			set
			{
				if (BaseGet(index) != null)
				{
					BaseRemoveAt(index);
				}
				BaseAdd(index, value);
			}
		}

		/// <summary>HttpCompressionModuleMimeTypeConfig</summary>
		new public ScriptAideReplacementRuleConfig this[string matchExpression]
		{
			get
			{
				return (ScriptAideReplacementRuleConfig)BaseGet(matchExpression);
			}
		}

		/// <summary>HttpCompressionModuleMimeTypeConfig</summary>
		public ScriptAideReplacementRuleConfigCollection()
		{
		}

		/// <summary>Add</summary>
		public void Add(ScriptAideReplacementRuleConfig item)
		{
			if (item == null)
				return;

			BaseAdd(item);
		}

		/// <summary>Clear</summary>
		public void Clear()
		{
			BaseClear();
		}

		/// <summary>Contains</summary>
		public bool Contains(ScriptAideReplacementRuleConfig item)
		{
			if (BaseIndexOf(item) >= 0)
				return true;
			else
				return false;
		}

		/// <summary>CopyTo</summary>
		public void CopyTo(ScriptAideReplacementRuleConfig[] array, int arrayIndex)
		{
			((ICollection<ScriptAideReplacementRuleConfig>)this).CopyTo(array, arrayIndex);
		}

		/// <summary>CreateNewElement</summary>
		protected override ConfigurationElement CreateNewElement()
		{
			return new ScriptAideReplacementRuleConfig();
		}

		/// <summary>IndexOf</summary>
		public int IndexOf(ScriptAideReplacementRuleConfig item)
		{
			if (item == null)
				return -1;

			return BaseIndexOf(item);
		}

		/// <summary>GetElementKey</summary>
		protected override object GetElementKey(ConfigurationElement element)
		{
			return ((ScriptAideReplacementRuleConfig)element).MatchExpression;
		}

		/// <summary>GetEnumerator</summary>
		public new IEnumerator<ScriptAideReplacementRuleConfig> GetEnumerator()
		{
			for (int i = 0; i < Count; i++)
			{
				yield return (ScriptAideReplacementRuleConfig)this[i];
			}
		}

		/// <summary>Remove</summary>
		public bool Remove(ScriptAideReplacementRuleConfig item)
		{
			if (item == null)
				return false;

			if (BaseIndexOf(item) >= 0)
				BaseRemove(item.MatchExpression);

			return true;
		}

		/// <summary>Remove</summary>
		public void Remove(string matchExpression)
		{
			BaseRemove(matchExpression);
		}

		/// <summary>RemoveAt</summary>
		public void RemoveAt(int index)
		{
			BaseRemoveAt(index);
		}

	}
}
