//------------------------------------------------------------------------------
// Copyright (c) 2008 www.dnaide.com
// Licensed under the MIT (MIT-LICENSE.txt)
//------------------------------------------------------------------------------
using System;
using System.Globalization;

namespace DNAide.Core
{
	public static partial class CastAide
	{

		/// <summary>
		/// Returns an object as a Int16.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <returns>Int16</returns>
		/// <remarks>
		/// <para>
		/// Will return 0 if the cast fails.
		/// </para>
		/// <para>
		/// Uses CultureInfo.CurrentCulture.
		/// </para>
		/// </remarks>
		public static Int16 AsInt16(object value)
		{
			return AsInt16(value, 0, CultureInfo.CurrentCulture);
		}


		/// <summary>
		/// Returns an object as a Int16.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <returns>Int16</returns>
		/// <remarks>
		/// <para>
		/// Uses CultureInfo.CurrentCulture.
		/// </para>
		/// </remarks>
		public static Int16 AsInt16(object value, Int16 defaultValue)
		{
			return AsInt16(value, defaultValue, CultureInfo.CurrentCulture);
		}


		/// <summary>
		/// Returns an object as a Int16.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="provider">Instance of IFormatProvider</param>
		/// <returns>Int16</returns>
		/// <remarks>
		/// <para>
		/// Will return 0 if the cast fails.
		/// </para>
		/// </remarks>
		public static Int16 AsInt16(object value, IFormatProvider provider)
		{
			return AsInt16(value, 0, provider);
		}


		/// <summary>
		/// Returns an object as a Int16.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <param name="provider">Instance of IFormatProvider</param>
		/// <returns>Int16</returns>
		public static Int16 AsInt16(object value, Int16 defaultValue, IFormatProvider provider)
		{
			if (value == null || value == System.DBNull.Value)
				return defaultValue;

			if (provider == null)
				provider = CultureInfo.CurrentCulture;

			Int16 retval;

			try
			{
				retval = Convert.ToInt16(value, provider);
			}
			catch (InvalidCastException)
			{
				retval = defaultValue;
			}
			catch (FormatException)
			{
				retval = defaultValue;
			}
			return retval;
		}


		/// <summary>
		/// Returns an object as a Int16 using the invariant culture.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <returns>Int16</returns>
		/// <remarks>
		/// <para>
		/// Will return 0 if the cast fails.
		/// </para>
		/// </remarks>
		public static Int16 AsInt16Invariant(object value)
		{
			return AsInt16(value, 0, CultureInfo.InvariantCulture);
		}


		/// <summary>
		/// Returns an object as a Int16 using the invariant culture.
		/// </summary>
		/// <param name="value">Value to cast</param>
		/// <param name="defaultValue">Value returned if cast fails</param>
		/// <returns>Int16</returns>
		public static Int16 AsInt16Invariant(object value, Int16 defaultValue)
		{
			return AsInt16(value, defaultValue, CultureInfo.InvariantCulture);
		}


	}
}
