//------------------------------------------------------------------------------
// Copyright (c) 2008 www.dnaide.com
// Licensed under the MIT (MIT-LICENSE.txt)
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using System.Globalization;
using System.Security.Cryptography;
using System.Text.RegularExpressions;
using System.Collections.Specialized;

namespace DNAide.Core
{
	public static partial class StringAide
	{

		/// <summary>
		/// Removes MS Word Html
		/// </summary>
		/// <param name="value">value</param>
		/// <returns>String</returns>
		public static String StripMSWordHtml(String value)
		{
			if (string.IsNullOrEmpty(value))
			{
				return string.Empty;
			}

			StringCollection sc = new StringCollection();
			
			// Get rid of office classes
			sc.Add(@"\s?class=[""']?Mso\w+[""']?");
			// get rid of unnecessary tag spans (comments and title)
			sc.Add(@"<!--(\w|\W)+?-->");
			// get rid of inline style
			sc.Add(@"\s?style=[""']?\w+[""']?");
			// Get rid of unnecessary tags
			sc.Add(@"<(meta|link|/?o:|/?style|/?font|/?st\d|/?head|/?html|body|/?body|!\[)[^>]*?>");
			// Get rid of empty tags (except table cells)
			sc.Add(@"(<[^/][^(th|d)>]*>){1}(&nbsp;)*(</[^>]+>){1}");
			// remove bizarre v: element attached to <img> tag
			sc.Add(@"\s+v:\w+=""[^""]+""");
			// remove extra lines
			sc.Add(@"(" + Environment.NewLine + "){2,}");
			// remove extra spaces
			sc.Add(@"( ){2,}");
			// remove font tags
			sc.Add(@"<font (.*?)>");
			sc.Add(@"</font>");
			foreach (string s in sc)
			{
				value = Regex.Replace(value, s, "", RegexOptions.IgnoreCase);
			}

			NameValueCollection nvc = new NameValueCollection();
			nvc.Add("\"", "&ldquo;");
			nvc.Add("\"", "&rdquo;");
			nvc.Add("", "&mdash;");
			foreach (string key in nvc.Keys)
			{
				value = value.Replace(key, nvc[key]);
			}

			return value;
		}


		/// <summary>
		/// Removes Xml from a string
		/// </summary>
		/// <remarks>Can be used to remove Html</remarks>
		/// <param name="value">value</param>
		/// <returns>String</returns>
		public static String StripXml(String value)
		{
			if (string.IsNullOrEmpty(value))
			{
				return string.Empty;
			}
			return Regex.Replace(value, @"<(.|\n)*?>", string.Empty);
		}


		/// <summary>
		/// Removes Xml from a string and trims whitespace
		/// </summary>
		/// <param name="value">value</param>
		/// <returns>String</returns>
		public static String StripXmlAndTrim(String value)
		{
			return StripXmlAndTrim(value, 0);
		}


		/// <summary>
		/// Removes Xml from a string and trims whitespace
		/// Also can sepcify a max length for the return string
		/// </summary>
		/// <param name="value">value</param>
		/// <param name="maxLength">maxLength</param>
		/// <returns>String</returns>
		public static String StripXmlAndTrim(String value, Int32 maxLength)
		{
			if (string.IsNullOrEmpty(value))
			{
				return string.Empty;
			}
			value = StripXml(value).Trim();
			if (maxLength > 0 && value.Length > maxLength)
			{
				value = Left(value, maxLength);
			}
			return value;
		}

		/// <summary>
		/// Removes Non-Alphanumeric characters from a string
		/// </summary>
		/// <param name="value">value</param>
		/// <returns>String</returns>
		public static String StripNonAlphaNumericCharacters(String value)
		{
			if (string.IsNullOrEmpty(value))
			{
				return string.Empty;
			}
			return Regex.Replace(value, "[^a-z0-9]", string.Empty, RegexOptions.IgnoreCase);
		}
	}
}
