<%@ Control Language="C#" AutoEventWireup="true" CodeBehind="ImageManipulation.ascx.cs" Inherits="DNAide.Website.UserControls.ImageManipulation" %>

<asp:PlaceHolder ID="ImageManipulationResizingPlh" runat="server" Visible="false">

<h2>ImageResize. <span>WebControl</span></h2>
<p>DNAide provides the <strong>ImageResize</strong> WebControl for resizing your images. Below are some examples of using this control.</p>

<h3>Original Image.</h3>
<img src="~/Client/Images/Mountain.jpg" width="400" height="300" />

<h3>Resized to 200x150 maintaining aspect ratio.</h3>

<pre class="csharpcode">
&lt;dnaide:ImageResize ID=<span class="str">"ImageResize1"</span> runat=<span class="str">"server"</span> SourceFileName=<span class="str">"~/Client/Images/MountainFullQuality.jpg"</span> NewWidth=<span class="str">"200"</span> NewHeight=<span class="str">"150"</span> OutputFileName=<span class="str">"ImageResize1.jpg"</span> /&gt;</pre>

<dnaide:ImageResize ID="ImageResize1" runat="server" SourceFileName="~/Client/Images/MountainFullQuality.jpg" NewWidth="200" NewHeight="150" OutputFileName="ImageResize1.jpg" />


<h3>Resized to 150x150 exact.</h3>

<pre class="csharpcode">
&lt;dnaide:ImageResize ID=<span class="str">"ImageResize2"</span> runat=<span class="str">"server"</span> SourceFileName=<span class="str">"~/Client/Images/MountainFullQuality.jpg"</span> NewWidth=<span class="str">"150"</span> NewHeight=<span class="str">"150"</span> MaintainAspectRatio=<span class="str">"false"</span> OutputFileName=<span class="str">"ImageResize2.jpg"</span> /&gt;</pre>

<dnaide:ImageResize ID="ImageResize2" runat="server" SourceFileName="~/Client/Images/MountainFullQuality.jpg" NewWidth="150" NewHeight="150" MaintainAspectRatio="false" OutputFileName="ImageResize2.jpg" />


<h3>Maintaining aspect ratio and saving as a 64 color gif.</h3>

<pre class="csharpcode">
&lt;dnaide:ImageResize ID=<span class="str">"ImageResize4"</span> runat=<span class="str">"server"</span> SourceFileName=<span class="str">"~/Client/Images/MountainFullQuality.jpg"</span> NewWidth=<span class="str">"150"</span> NewHeight=<span class="str">"150"</span> OutputFileName=<span class="str">"ImageResize4.gif"</span> OutputGifMaxColors=<span class="str">"64"</span> /&gt;</pre>

<dnaide:ImageResize ID="ImageResize4" runat="server" SourceFileName="~/Client/Images/MountainFullQuality.jpg" NewWidth="150" NewHeight="150" OutputFileName="ImageResize4.gif" OutputGifMaxColors="64" />

<p>NB. Saving your output images as Gifs uses a Quantization process to achive a better quality Gif. This Quantization is based on original code by <a href="http://codebetter.com/blogs/brendan.tompkins/archive/2007/06/14/gif-image-color-quantizer-now-with-safe-goodness.aspx" class="newwindow">Brendan Tompkins</a>.</p>

</asp:PlaceHolder>



<asp:PlaceHolder ID="ImageManipulationFiltersPlh" runat="server" Visible="false">

<h2>ImageTransform <span>The Daddy Control.</span></h2>
<p>The <strong>ImageTransform</strong> WebControl allows you to apply various effects and filters on your images.</p>
<p>This WebControl is a helper and wrapper for the filter/transform methods in the <strong>DNAide.Drawing.BitmapAide</strong> class. In fact, the <a href="resizing-images.htm">ImageResize</a> WebControl inherits from this WebControl.</p>
<p>The WebControl raises an event allowing you to apply any number of filters and transformations to your image before saving the final output.</p>
<p>To avoid repitition in the examples below, only the first filter "Grayscale", shows both aspx and codebehind. Subsequent examples show the codebehind event handler code only.</p>
<p>These filters are based on the articles written by Christian Graus in his excellent series "Image Processing for Dummies with C# and GDI+" published on <a href="http://www.codeproject.com/cs/media/csharpgraphicfilters11.asp" class="newwindow">The Code Project.</a></p>

<h3>Original Image.</h3>
<img src="~/Client/Images/Matches.jpg" width="300" height="193" />

<h3>Grayscale.</h3>

<p><strong>ASPX:</strong></p>
<pre class="csharpcode">
&lt;dnaide:ImageTransform ID=<span class="str">"ImageTransform1"</span> runat=<span class="str">"server"</span> SourceFileName=<span class="str">"~/Client/Images/Matches.jpg"</span> OutputFileName=<span class="str">"ImageTransform1.jpg"</span> /&gt;</pre>

<p><strong>CodeBehind:</strong></p>

<pre class="csharpcode">
<span class="kwrd">protected</span> <span class="kwrd">override</span> <span class="kwrd">void</span> OnInit(EventArgs e)
{
  <span class="kwrd">this</span>.ImageTransform1.ApplyTransformations += 
    <span class="kwrd">new</span> DNAideImageControl.ApplyTransformationsEventHandler (ImageTransform1_ApplyTransformations);
  <span class="kwrd">base</span>.OnInit(e);
}

<span class="kwrd">void</span> ImageTransform1_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  BitmapAide.GrayScale(bitmapToTransform);
}</pre>
        
<dnaide:ImageTransform ID="ImageTransform1" runat="server" SourceFileName="~/Client/Images/Matches.jpg" OutputFileName="ImageTransform1.jpg" />


<h3>Brightness.</h3>

<pre class="csharpcode">
<span class="kwrd">void</span> ImageTransform2_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  BitmapAide.Brightness(bitmapToTransform, 100);
}</pre>

<dnaide:ImageTransform ID="ImageTransform2" runat="server" SourceFileName="~/Client/Images/Matches.jpg" OutputFileName="ImageTransform2.jpg" />


<h3>Contrast.</h3>

<pre class="csharpcode">
<span class="kwrd">void</span> ImageTransform3_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  BitmapAide.Contrast(bitmapToTransform, 100);
}</pre>

<dnaide:ImageTransform ID="ImageTransform3" runat="server" SourceFileName="~/Client/Images/Matches.jpg" OutputFileName="ImageTransform3.jpg" />


<h3>Sharpen.</h3>

<pre class="csharpcode">
<span class="kwrd">void</span> ImageTransform4_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  BitmapAide.Sharpen(bitmapToTransform, 11);
}</pre>

<dnaide:ImageTransform ID="ImageTransform4" runat="server" SourceFileName="~/Client/Images/Matches.jpg" OutputFileName="ImageTransform4.jpg" />


<h3>Smooth.</h3>

<pre class="csharpcode">
<span class="kwrd">void</span> ImageTransform5_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  BitmapAide.Smooth(bitmapToTransform, 11);
}</pre>

<dnaide:ImageTransform ID="ImageTransform5" runat="server" SourceFileName="~/Client/Images/Matches.jpg" OutputFileName="ImageTransform5.jpg" />


<h3>GaussianBlur.</h3>

<pre class="csharpcode">
<span class="kwrd">void</span> ImageTransform6_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  BitmapAide.GaussianBlur(bitmapToTransform, 4);
}</pre>

<dnaide:ImageTransform ID="ImageTransform6" runat="server" SourceFileName="~/Client/Images/Matches.jpg" OutputFileName="ImageTransform6.jpg" />


<h3>Emboss.</h3>

<pre class="csharpcode">
<span class="kwrd">void</span> ImageTransform7_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  BitmapAide.EmbossLaplacian(bitmapToTransform);
}</pre>

<dnaide:ImageTransform ID="ImageTransform7" runat="server" SourceFileName="~/Client/Images/Matches.jpg" OutputFileName="ImageTransform7.jpg" />


<h3>Pixellate.</h3>

<pre class="csharpcode">
<span class="kwrd">void</span> ImageTransform8_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  BitmapAide.Pixellate(bitmapToTransform, 10, true);
}</pre>

<dnaide:ImageTransform ID="ImageTransform8" runat="server" SourceFileName="~/Client/Images/Matches.jpg" OutputFileName="ImageTransform8.jpg" />


<h3>Invert.</h3>

<pre class="csharpcode">
<span class="kwrd">void</span> ImageTransform9_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  BitmapAide.Invert(bitmapToTransform);
}</pre>

<dnaide:ImageTransform ID="ImageTransform9" runat="server" SourceFileName="~/Client/Images/Matches.jpg" OutputFileName="ImageTransform9.jpg" />


<h3>Water.</h3>

<pre class="csharpcode">
<span class="kwrd">void</span> ImageTransform10_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  BitmapAide.Water(bitmapToTransform, 10, true);
}</pre>

<dnaide:ImageTransform ID="ImageTransform10" runat="server" SourceFileName="~/Client/Images/Matches.jpg" OutputFileName="ImageTransform10.jpg" />

</asp:PlaceHolder>



<asp:PlaceHolder ID="ImageManipulationRotatingPlh" runat="server" Visible="false">

<h2>Rotating Your Images. <span>Use the ImageTransform WebControl!</span></h2>
<p>It's pretty easy to rotate your images. Simply use the ImageTransform WebControl, capture the <strong>ApplyTransformations</strong> event, and perform a rotate (see article on <a href="image-filters.htm">Image Filters</a> on a quick low down).</p>

<h3>Original Image.</h3>
<img src="~/Client/Images/Building.jpg" width="300" height="225" />

<h2>Rotated 90 degrees.</h2>
<pre class="csharpcode">
<span class="kwrd">void</span> ImageRotate1_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  bitmapToTransform.RotateFlip(RotateFlipType.Rotate90FlipNone);
}</pre>

<dnaide:ImageTransform ID="ImageRotate1" runat="server" SourceFileName="~/Client/Images/Building.jpg" OutputFileName="ImageRotate1.jpg" />

</asp:PlaceHolder>




<asp:PlaceHolder ID="ImageManipulationCroppingPlh" runat="server" Visible="false">

<h2>Cropping Your Images.</h2>
<p>As with the previous two articles, you can crop your images using the <strong>ImageTransform</strong> WebControl and capturing the <strong>ApplyTransformations</strong> event to performing a crop.</p>

<h3>Original Image.</h3>
<img src="~/Client/Images/Egypt.jpg" width="300" height="190" />

<h3>Cropped Image.</h3>

<pre class="csharpcode">
<span class="kwrd">void</span> ImageCrop1_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
  BitmapAide.Crop(ref bitmapToTransform, 131, 91, 50, 50);
}</pre>

<dnaide:ImageTransform ID="ImageCrop1" runat="server" SourceFileName="~/Client/Images/Egypt.jpg" OutputFileName="ImageCrop1.jpg" />

</asp:PlaceHolder>




<asp:PlaceHolder ID="ImageManipulationTextPlh" runat="server" Visible="false">

<h2>Creating Text Based Images.</h2>
<p>DNAide comes with the <strong>ImageText</strong> control. This control allows you to render dynamic text based images on the fly.</p>

<p>By default, the control generates your image and renders a XHTML &lt;img&gt; tag. This is fine for simple scenarios. However, the control also provides a simple JavaScript based image replacement technique.</p>
<p>What this means is that you provide the XHTML tag and copy as part of the control e.g. &lt;h2&gt;My Header&lt;/h2&gt;. On render, the control will also generate a simple JavaScript command which, when run by your browser, will replace your tag's copy with an image tag i.e. the final XHTML after the JavaScript has run will be something similar to &lt;h2&gt;&lt;img src="my-header.gif" alt="My Header" /&gt;&lt;/h2&gt;.</p>

<p>In addition, the <strong>ImageText</strong> control, like the <strong>ImageResize</strong> control inherits from the <strong>ImageTransform</strong> control which provides you with the option to apply any effect/filter onto your text based image.</p>


<h3>Simple Text Based Image.</h3>

<pre class="csharpcode">
&lt;dnaide:ImageText runat=<span class="str">"server"</span> Text=<span class="str">"Hello World"</span> FontSize=<span class="str">"25"</span> ForeColor=<span class="str">"Green"</span> /&gt;</pre>

<dnaide:ImageText ID="ImageText1" runat="server" Text="Hello World" FontSize="25" ForeColor="Green" OutputFileName="ImageText1.gif" />


<h3>Using Installed Fonts.</h3>
<p>If you don't specify a font family, the control will by default generate an image using Arial.</p>
<pre class="csharpcode">
&lt;dnaide:ImageText ID=<span class="str">"ImageText2"</span> runat=<span class="str">"server"</span> Text=<span class="str">"Hello World Pt2"</span> FontSize=<span class="str">"25"</span> ForeColor=<span class="str">"Blue"</span> FontFamily=<span class="str">"Comic Sans MS"</span>  /&gt;</pre>

<dnaide:ImageText ID="ImageText2" runat="server" Text="Hello World Pt2" FontSize="25" ForeColor="Blue" FontFamily="Comic Sans MS" OutputFileName="ImageText2.gif"  />


<h3>Using Font Files.</h3>
<p>You don't have to use installed fonts. You can specify the path to a TrueType font file.</p>
<pre class="csharpcode">
&lt;dnaide:ImageText ID=<span class="str">"ImageText3"</span> runat=<span class="str">"server"</span> Text=<span class="str">"Hello World Pt3"</span> FontSize=<span class="str">"25"</span> ForeColor=<span class="str">"Black"</span> FontFileName=<span class="str">"~/Fonts/caeldera.ttf"</span> /&gt;</pre>

<dnaide:ImageText ID="ImageText3" runat="server" Text="Hello World Pt3" FontSize="25" ForeColor="Black" FontFileName="~/Fonts/caeldera.ttf" OutputFileName="ImageText3.gif" />


<h3>Image Replacement Technique.</h3>
<pre class="csharpcode">
&lt;dnaide:ImageText ID=<span class="str">"ImageText4"</span> runat=<span class="str">"server"</span> FontSize=<span class="str">"25"</span> ForeColor=<span class="str">"Red"</span> RenderType=<span class="str">"ClientScriptReplace"</span>&gt;
    &lt;h4 id=<span class="str">"##DNAideImageTextId##"</span>&gt;Hello World Pt4&lt;/h4&gt;
&lt;/dnaide:ImageText&gt;</pre>

<dnaide:ImageText ID="ImageText4" runat="server" FontSize="25" ForeColor="Red" RenderType="ClientScriptReplace" OutputFileName="ImageText4.gif">
	<h4 id="##DNAideImageTextId##">Hello World Pt4</h4>
</dnaide:ImageText>


<h3>Long Text and Wrapping.</h3>
<pre class="csharpcode">
&lt;dnaide:ImageText ID=<span class="str">"ImageText5"</span> runat=<span class="str">"server"</span> Text=<span class="str">"Hello World Pt3. It contains some text that goes on and on and on and on and on"</span> FontSize=<span class="str">"25"</span> ForeColor=<span class="str">"Black"</span> MaxWidth=<span class="str">"400"</span> /&gt;
</pre>
<dnaide:ImageText ID="ImageText5" runat="server" Text="Hello World Pt5. It contains some text that goes on and on and on and on and on" FontSize="25" ForeColor="Black" MaxWidth="400" OutputFileName="ImageText5.gif" />


<h3>Font Smoothing / Anti-Aliasing</h3>
<p>By default, the control uses your system's font smoothing settings. This is fine if your development environment and live are identical. Chances are this will not be the case. To remedy this, the control allows you to explicitly set the <strong>TextRenderingHint</strong>.</p>
<pre class="csharpcode">
&lt;dnaide:ImageText ID=<span class="str">"ImageText6"</span> runat=<span class="str">"server"</span> Text=<span class="str">"Hello World Pt6"</span> FontSize=<span class="str">"25"</span> TextRenderingHint=<span class="str">"ClearTypeGridFit"</span> /&gt;</pre>

<dnaide:ImageText ID="ImageText6" runat="server" Text="Hello World Pt6" FontSize="25" TextRenderingHint="ClearTypeGridFit" />


<h3>Applying Effects etc.</h3>
<p>The example below applies a pixellate filter on the text image. This is applied using by capturing the <strong>ApplyTransformations</strong> event.</p>

<p><strong>ASPX:</strong></p>
<pre class="csharpcode">
&lt;dnaide:ImageText ID=<span class="str">"ImageText7"</span> runat=<span class="str">"server"</span> Text=<span class="str">"Hello World Pt7"</span> FontSize=<span class="str">"25"</span> ForeColor=<span class="str">"Red"</span> /&gt;</pre>

<p><strong>CodeBehind:</strong></p>
<pre class="csharpcode">
<span class="kwrd">void</span> ImageText7_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
    BitmapAide.Pixellate(bitmapToTransform, 3, true);
}</pre>

<dnaide:ImageText ID="ImageText7" runat="server" Text="Hello World Pt7" FontSize="25" ForeColor="Red" OutputFileName="ImageText7.gif" />

</asp:PlaceHolder>




<asp:PlaceHolder ID="ImageManipulationDrawingOnToCanvasPlh" runat="server" Visible="false">

<h2>Overlaying Images <span>And Watermarking Made Easy!</span></h2>

<p>By capturing the <strong>ApplyTransformations</strong> event in any of the DNAide image controls, you can overlay images on top of one another and specify positionning etc.</p>
<p>This functionality can be used to make complex composite images or make watermarking your images easy!</p>

<h3>Simple Overlaying.</h3>

<p><strong>ASPX:</strong></p>
<pre class="csharpcode">
&lt;dnaide:ImageTransform ID=<span class="str">"ImageCanvas1"</span> runat=<span class="str">"server"</span> SourceFileName=<span class="str">"~/Client/Images/Frame.jpg"</span> /&gt; </pre>

<p><strong>CodeBehind:</strong></p>
<pre class="csharpcode">
<span class="kwrd">void</span> ImageCanvas1_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
    Bitmap picture = BitmapAide.Load(Server.MapPath(<span class="str">"~/Client/Images/FramePicture.jpg"</span>));
    BitmapAide.DrawOnToCanvasBitmap(<span class="kwrd">ref</span> bitmapToTransform, picture, DrawPosition.CenterCenter);
    picture.Dispose();
}</pre>

<dnaide:ImageTransform ID="ImageCanvas1" runat="server" SourceFileName="~/Client/Images/Frame.jpg" OutputFileName="ImageCanvas1.jpg" /> 


<h3>Watermarking.</h3>
<p>The example below uses a PNG as the watermark image with a 50% opacity.</p>

<p><strong>ASPX:</strong></p>
<pre class="csharpcode">
&lt;dnaide:ImageTransform ID=<span class="str">"ImageCanvas2"</span> runat=<span class="str">"server"</span> SourceFileName=<span class="str">"~/Client/Images/Mountain.jpg"</span> /&gt; </pre>

<p><strong>CodeBehind:</strong></p>
<pre class="csharpcode">
<span class="kwrd">void</span> ImageCanvas2_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
    Bitmap watermark = BitmapAide.Load(Server.MapPath(<span class="str">"~/Client/Images/WaterMark.png"</span>));
    BitmapAide.DrawOnToCanvasBitmap(<span class="kwrd">ref</span> bitmapToTransform, watermark, DrawPosition.BottomRight);
    watermark.Dispose();
}</pre>

<dnaide:ImageTransform ID="ImageCanvas2" runat="server" SourceFileName="~/Client/Images/Mountain.jpg" OutputFileName="ImageCanvas2.jpg" /> 


</asp:PlaceHolder>




<asp:PlaceHolder ID="ImageManipulationCombiningEffectsPlh" runat="server" Visible="false">
<h2>Making Something...<span>Er Funky!</span></h2>
<p>Like in previous examples, you can use the <strong>ApplyTransformations</strong> event in any of the DNAide image controls to combine multiple transformations on your image.</strong></p>
<p>The example below isn't pretty, but shows combining the following: "GrayScale", "Water", and "RotateFlip" to an image.</p>

<p><strong>CodeBehind:</strong></p>
<pre class="csharpcode">
<span class="kwrd">void</span> ImageTransform11_ApplyTransformations(<span class="kwrd">object</span> sender, <span class="kwrd">ref</span> Bitmap bitmapToTransform)
{
    BitmapAide.GrayScale(bitmapToTransform);
    BitmapAide.Water(bitmapToTransform, 10, <span class="kwrd">true</span>);
    bitmapToTransform.RotateFlip(RotateFlipType.Rotate90FlipNone);
}</pre>

<dnaide:ImageTransform ID="ImageTransform11" runat="server" SourceFileName="~/Client/Images/Mountain.jpg" OutputFileName="ImageTransform11.jpg" /> 

</asp:PlaceHolder>